/***********************************************************************
*                                                                      *
*      Include files.                                                  *
*                                                                      *
***********************************************************************/
/* Device header file */
#if defined(__PIC24E__)
  #include <p24Exxxx.h>
#elif defined(__PIC24F__)
	#include <p24Fxxxx.h>
#elif defined(__PIC24H__)
	#include <p24Hxxxx.h>
#endif

#include <stdint.h>        // Includes uint16_t definition.
#include <stdbool.h>       // Includes true/false definition.

#include "system.h"        // System funct/params, like osc/peripheral config.
#include "user.h"          // User funct/params, such as InitApp.
#include "colors.h"        // HTML color constants.

/***********************************************************************
*                                                                      *
*      Global variable declaration.                                    *
*                                                                      *
***********************************************************************/
// i.e. uint16_t <variable_name>;
int indicator = 0;    // Currently selected indicator led nbr.

// pwm[] table contains current pwm values (0..1023) for each pwm
// channel (0..23).
int pwm[]=
{
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};


// Gamma table to translate visual percentage to actual led percentage.
unsigned int g[] =
{
  0, 2, 3, 5, 7, 8, 9, 10, 11, 11, 11, 12, 13, 14, 15, 16,
  17, 19, 23, 23, 27, 27, 31, 35, 35, 39, 43, 47, 51, 55, 59, 63,
  71, 75, 83, 91, 99, 107, 115, 127, 139, 151, 163, 179, 195, 211, 231, 251,
  275, 299, 327, 359, 391, 427, 467, 507, 555, 603, 659, 719, 787, 859, 939, 1023
};

int program = 0;  // Currently selected program.

// firstTime used for all programs, to do first time initialisations, reset by
// main program.
int firstTime = 1;

int red1, green1, blue1, red1New, green1New, blue1New;
int red2, green2, blue2, red2New, green2New, blue2New;
int red3, green3, blue3, red3New, green3New, blue3New;
int red4, green4, blue4, red4New, green4New, blue4New;
int red5, green5, blue5, red5New, green5New, blue5New;
int red6, green6, blue6, red6New, green6New, blue6New;
int red7, green7, blue7, red7New, green7New, blue7New;
int red8, green8, blue8, red8New, green8New, blue8New;

int red, green, blue, redNew, greenNew, blueNew;

/***********************************************************************
*                                                                      *
*      Function prototypes                                             *
*                                                                      *
***********************************************************************/
int program0();
int program1();
int program2();
int program3();
int program4();
int program5();
int program6();
int program7();
int program8();
int program9();
int program10();
int program11();
int program12();
int program13();
int program14();
int program15();
int program16();
int program17();
int program18();
int program19();
int program20();
int program21();
int program22();
int program23();
int program24();
int program25();
int program26();
int program27();
int program28();
int program29();
int program30();
int program31();

/***********************************************************************
*                                                                      *
*      01  Main program                                                *
*                                                                      *
***********************************************************************/
int16_t main(void)
{

  // Configure the oscillator for the device.
  ConfigureOscillator();

  // Initialize IO ports and peripherals.
  InitApp();
  /* USER APPLICATION CODE HERE */

  int i, p = 0, pt = 0;

  // Test the indicator led bar.
  for (i = 0; i < 32; i++)
  {
    indicator = i;
    delay_ms(10);
  }
  delay_ms(1000);
  indicator = 0;

  // Before main loop, first some testing.
  // Soft start of LED1.
  while (p < MAX)
  {
    p++;                                 // Increment pointer to look-up table.
    pt = g[p];                           // Translate pointer into gamma-value.
    SetPwm(PORT_LED1R, pt);              // Dim LED accordingly.
    SetPwm(PORT_LED1G, pt);              // Dim LED accordingly.
    SetPwm(PORT_LED1B, pt);              // Dim LED accordingly.
    delay_ms(SOFT_START);
  }


  // Set all LEDS on some predefined colors and levels.
  SetColor(1, orange4);
  SetColor(2, khaki4);
  SetColor(3, seagreen1);
  SetColor(4, sandybrown);
  SetColor(5, limegreen);
  SetColor(6, medium_forest_green);
  SetColor(7, coral3);
  SetColor(8, red_);

  delay_ms(5000);


  // Main loop.
  while (1)
  {
    ClrWdt();
    indicator = program;  // Reflect choosen program on LED indicator display.

    if (program == 0) program0();
    if (program == 1) program1();
    if (program == 2) program2();
    if (program == 3) program3();
    if (program == 4) program4();
    if (program == 5) program5();
    if (program == 6) program6();
    if (program == 7) program7();
    if (program == 8) program8();
    if (program == 9) program9();
    if (program == 10) program10();
    if (program == 11) program11();
    if (program == 12) program12();
    if (program == 13) program0();
    if (program == 14) program1();
    if (program == 15) program2();
    if (program == 16) program3();
    if (program == 17) program4();
    if (program == 18) program5();
    if (program == 19) program6();
    if (program == 20) program7();
    if (program == 21) program8();
    if (program == 22) program9();
    if (program == 23) program10();
    if (program == 24) program11();
    if (program == 25) program12();
    if (program == 26) program0();
    if (program == 27) program1();
    if (program == 28) program2();
    if (program == 29) program3();
    if (program == 30) program4();
    if (program == 31) program31();

    if (PUSHBUTTON == 0 && !DEBUG)   // If button pressed...
    {
      Delay_Us(30000);     // Debounce.
      if (PUSHBUTTON == 0) // If button pressed...
      {
        while (PUSHBUTTON == 0); // Wait for button to release.
        program++;               // Next program.
        if (program > 31) program = 0;
        Delay_Us(10000);
        firstTime = 1;           // Make sure next program
                                 // will initialize.
      }
    }
  }

  return 0;
}

/***********************************************************************
*                                                                      *
*      03  program0                    parent = 01  main               *
*                                                                      *
*      Random colors and random intensity for each LED, 1 by 1,        *
*      going from LED1 to LED8 in order and with fade-over effect.     *
*                                                                      *
***********************************************************************/
int program0()
{
// Max brighness for this program.
#define BRIGHTNESS_0 44

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    red1 = rand() % BRIGHTNESS_0;
    green1 = rand() % BRIGHTNESS_0;
    blue1 = rand() % BRIGHTNESS_0;
    red2 = rand() % BRIGHTNESS_0;
    green2 = rand() % BRIGHTNESS_0;
    blue2 = rand() % BRIGHTNESS_0;
    red3 = rand() % BRIGHTNESS_0;
    green3 = rand() % BRIGHTNESS_0;
    blue3 = rand() % BRIGHTNESS_0;
    red4 = rand() % BRIGHTNESS_0;
    green4 = rand() % BRIGHTNESS_0;
    blue4 = rand() % BRIGHTNESS_0;
    red5 = rand() % BRIGHTNESS_0;
    green5 = rand() % BRIGHTNESS_0;
    blue5 = rand() % BRIGHTNESS_0;
    red6 = rand() % BRIGHTNESS_0;
    green6 = rand() % BRIGHTNESS_0;
    blue6 = rand() % BRIGHTNESS_0;
    red7 = rand() % BRIGHTNESS_0;
    green7 = rand() % BRIGHTNESS_0;
    blue7 = rand() % BRIGHTNESS_0;
    red8 = rand() % BRIGHTNESS_0;
    green8 = rand() % BRIGHTNESS_0;
    blue8 = rand() % BRIGHTNESS_0;
  }

  SetPwm(PORT_LED1R, g[red1]);
  SetPwm(PORT_LED1G, g[green1]);
  SetPwm(PORT_LED1B, g[blue1]);
  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  SetPwm(PORT_LED2R, g[red2]);
  SetPwm(PORT_LED2G, g[green2]);
  SetPwm(PORT_LED2B, g[blue2]);
  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  SetPwm(PORT_LED3R, g[red3]);
  SetPwm(PORT_LED3G, g[green3]);
  SetPwm(PORT_LED3B, g[blue3]);
  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  SetPwm(PORT_LED4R, g[red4]);
  SetPwm(PORT_LED4G, g[green4]);
  SetPwm(PORT_LED4B, g[blue4]);
  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  SetPwm(PORT_LED5R, g[red5]);
  SetPwm(PORT_LED5G, g[green5]);
  SetPwm(PORT_LED5B, g[blue5]);
  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  SetPwm(PORT_LED6R, g[red6]);
  SetPwm(PORT_LED6G, g[green6]);
  SetPwm(PORT_LED6B, g[blue6]);
  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  SetPwm(PORT_LED7R, g[red7]);
  SetPwm(PORT_LED7G, g[green7]);
  SetPwm(PORT_LED7B, g[blue7]);
  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  SetPwm(PORT_LED8R, g[red8]);
  SetPwm(PORT_LED8G, g[green8]);
  SetPwm(PORT_LED8B, g[blue8]);
  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;

  red1New = rand() % BRIGHTNESS_0;
  green1New = rand() % BRIGHTNESS_0;
  blue1New = rand() % BRIGHTNESS_0;
  red2New = rand() % BRIGHTNESS_0;
  green2New = rand() % BRIGHTNESS_0;
  blue2New = rand() % BRIGHTNESS_0;
  red3New = rand() % BRIGHTNESS_0;
  green3New = rand() % BRIGHTNESS_0;
  blue3New = rand() % BRIGHTNESS_0;
  red4New = rand() % BRIGHTNESS_0;
  green4New = rand() % BRIGHTNESS_0;
  blue4New = rand() % BRIGHTNESS_0;
  red4New = rand() % BRIGHTNESS_0;
  green4New = rand() % BRIGHTNESS_0;
  blue4New = rand() % BRIGHTNESS_0;
  red5New = rand() % BRIGHTNESS_0;
  green5New = rand() % BRIGHTNESS_0;
  blue5New = rand() % BRIGHTNESS_0;
  red6New = rand() % BRIGHTNESS_0;
  green6New = rand() % BRIGHTNESS_0;
  blue6New = rand() % BRIGHTNESS_0;
  red7New = rand() % BRIGHTNESS_0;
  green7New = rand() % BRIGHTNESS_0;
  blue7New = rand() % BRIGHTNESS_0;
  red8New = rand() % BRIGHTNESS_0;
  green8New = rand() % BRIGHTNESS_0;
  blue8New = rand() % BRIGHTNESS_0;

  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  FadeOver(PORT_LED1R, PORT_LED1G, PORT_LED1B,
           red1, green1, blue1,
           red1New, green1New, blue1New,
           1000);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  red1 = red1New;
  green1 = green1New;
  blue1 = blue1New;

  delay_ms(500);
  if (PUSHBUTTON == 0 && !DEBUG) return 0;
  FadeOver(PORT_LED2R, PORT_LED2G, PORT_LED2B,
           red2, green2, blue2,
           red2New, green2New, blue2New,
           1000);
  red2 = red2New;
  green2 = green2New;
  blue2 = blue2New;

  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;
  FadeOver(PORT_LED3R, PORT_LED3G, PORT_LED3B,
           red3, green3, blue3,
           red3New, green3New, blue3New,
           1000);
  red3 = red3New;
  green3 = green3New;
  blue3 = blue3New;

  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;
  FadeOver(PORT_LED4R, PORT_LED4G, PORT_LED4B,
           red4, green4, blue4,
           red4New, green4New, blue4New,
           1000);
  red4 = red4New;
  green4 = green4New;
  blue4 = blue4New;

  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;
  FadeOver(PORT_LED5R, PORT_LED5G, PORT_LED5B,
           red5, green5, blue5,
           red5New, green5New, blue5New,
           1000);
  red5 = red5New;
  green5 = green5New;
  blue5 = blue5New;

  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;
  FadeOver(PORT_LED6R, PORT_LED6G, PORT_LED6B,
           red6, green6, blue6,
           red6New, green6New, blue6New,
           1000);
  red6 = red6New;
  green6 = green6New;
  blue6 = blue6New;

  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;
  FadeOver(PORT_LED7R, PORT_LED7G, PORT_LED7B,
           red7, green7, blue7,
           red7New, green7New, blue7New,
           1000);
  red7 = red7New;
  green7 = green7New;
  blue7 = blue7New;

  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;
  FadeOver(PORT_LED8R, PORT_LED8G, PORT_LED8B,
           red8, green8, blue8,
           red8New, green8New, blue8New,
           1000);
  red8 = red8New;
  green8 = green8New;
  blue8 = blue8New;

  return 0;
}


/***********************************************************************
*                                                                      *
*      03  program1                    parent = 01  main               *
*                                                                      *
*      Fixed colors and fixed intensities, changing to a second set    *
*      of fixed colors and intensities.                                *
*                                                                      *
***********************************************************************/
int program1()
{
  SetColor(1, indianred4);
  SetColor(2, cyan4);
  SetColor(3, steelblue4);
  SetColor(4, saddlebrown);
  SetColor(5, yellow2);
  SetColor(6, rosybrown1);
  SetColor(7, slategray);
  SetColor(8, darkorange4);
  
  delay_ms(2000);
  if (PUSHBUTTON == 0) return 0;

  SetColor(1, lightcoral);
  SetColor(2, darkolivegreen1);
  SetColor(3, burlywood);
  SetColor(4, olivedrab);
  SetColor(5, green4_);
  SetColor(6, orange3);
  SetColor(7, red4_);
  SetColor(8, yellow4);

  delay_ms(2000);
  if (PUSHBUTTON == 0) return 0;

  return 0;
}

/***********************************************************************
*                                                                      *
*      03  program2                    parent = 01  main               *
*                                                                      *
*      Red only. Random led set to random intensity, then random led   *
*      set OFF.                                                        *
*                                                                      *
***********************************************************************/
int program2()
{
// Max brighness for this program.
#define BRIGHTNESS_2 36

  int randomLed, randomPort, randomValue;

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }
  randomLed = rand() % 8;
  randomPort = randomLed * 3;
  randomValue = rand() % BRIGHTNESS_2;
  SetPwm(randomPort, g[randomValue]);
  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;
  randomLed = rand() % 8;
  randomPort = randomLed * 3;
  SetPwm(randomPort, 0);
  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;

  return 0;

}


/***********************************************************************
*                                                                      *
*      03  program3                    parent = 01  main               *
*                                                                      *
*      Red only. Running light with fixed intensity.                   *
*                                                                      *
***********************************************************************/
int program3()
{
// Max brighness for this program.
#define BRIGHTNESS_3 40

  int i;

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }
  for (i = 0; i < NBR_LEDS; i += 3)
  {
    SetPwm(i, g[BRIGHTNESS_3]);
    delay_ms(100);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(i, 0);
    delay_ms(100);
    if (PUSHBUTTON == 0) return 0;
  }

  return 0;
}



/***********************************************************************
*                                                                      *
*      03  program4                    parent = 01  main               *
*                                                                      *
*      Green only. Running light with fixed intensity.                 *
*                                                                      *
***********************************************************************/
int program4()
{
// Max brighness for this program.
#define BRIGHTNESS_4 42

  int i;

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }
  for (i = 1; i < NBR_LEDS; i += 3)
  {
    SetPwm(i, g[BRIGHTNESS_4]);
    delay_ms(100);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(i, 0);
    delay_ms(100);
    if (PUSHBUTTON == 0) return 0;
  }

  return 0;
}



/***********************************************************************
*                                                                      *
*      03  program5                    parent = 01  main               *
*                                                                      *
*      Blue only. Running light with fixed intensity.                  *
*                                                                      *
***********************************************************************/
int program5()
{
// Max brighness for this program.
#define BRIGHTNESS_5 38

  int i;

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }
  for (i = 2; i < NBR_LEDS; i += 3)
  {
    SetPwm(i, g[BRIGHTNESS_5]);
    delay_ms(100);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(i, 0);
    delay_ms(100);
    if (PUSHBUTTON == 0) return 0;
  }

  return 0;
}




/***********************************************************************
*                                                                      *
*      03  program6                    parent = 01  main               *
*                                                                      *
*      White only. Running light with fixed intensity.                 *
*                                                                      *
***********************************************************************/
int program6()
{
// Max brighness for this program.
#define BRIGHTNESS_6 44

  int i;

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }
  for (i = 0; i < NBR_LEDS; i += 3)
  {
    SetPwm(i, g[BRIGHTNESS_6]);
    SetPwm(i +1, g[BRIGHTNESS_6]);
    SetPwm(i +2, g[BRIGHTNESS_6 -1]);
    delay_ms(100);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(i, 0);
    SetPwm(i +1, 0);
    SetPwm(i +2, 0);
    delay_ms(100);
    if (PUSHBUTTON == 0) return 0;
  }

  return 0;
}


/***********************************************************************
*                                                                      *
*      03  program7                    parent = 01  main               *
*                                                                      *
*      Random colors and random intensity for each LED, 1 by 1,        *
*      randomly choosen and with fade-over effect.                     *
*                                                                      *
***********************************************************************/
int program7()
{
// Max brighness for this program.
#define BRIGHTNESS_7 40

  int randomLed;

  randomLed = rand() % 8;
  
  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
    red1 = rand() % BRIGHTNESS_7;
    green1 = rand() % BRIGHTNESS_7;
    blue1 = rand() % BRIGHTNESS_7;
    red2 = rand() % BRIGHTNESS_7;
    green2 = rand() % BRIGHTNESS_7;
    blue2 = rand() % BRIGHTNESS_7;
    red3 = rand() % BRIGHTNESS_7;
    green3 = rand() % BRIGHTNESS_7;
    blue3 = rand() % BRIGHTNESS_7;
    red4 = rand() % BRIGHTNESS_7;
    green4 = rand() % BRIGHTNESS_7;
    blue4 = rand() % BRIGHTNESS_7;
    red5 = rand() % BRIGHTNESS_7;
    green5 = rand() % BRIGHTNESS_7;
    blue5 = rand() % BRIGHTNESS_7;
    red6 = rand() % BRIGHTNESS_7;
    green6 = rand() % BRIGHTNESS_7;
    blue6 = rand() % BRIGHTNESS_7;
    red7 = rand() % BRIGHTNESS_7;
    green7 = rand() % BRIGHTNESS_7;
    blue7 = rand() % BRIGHTNESS_7;
    red8 = rand() % BRIGHTNESS_7;
    green8 = rand() % BRIGHTNESS_7;
    blue8 = rand() % BRIGHTNESS_7;
    
    SetPwm(PORT_LED1R, g[red1]);
    SetPwm(PORT_LED1G, g[green1]);
    SetPwm(PORT_LED1B, g[blue1]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED2R, g[red2]);
    SetPwm(PORT_LED2G, g[green2]);
    SetPwm(PORT_LED2B, g[blue2]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED3R, g[red3]);
    SetPwm(PORT_LED3G, g[green3]);
    SetPwm(PORT_LED3B, g[blue3]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED4R, g[red4]);
    SetPwm(PORT_LED4G, g[green4]);
    SetPwm(PORT_LED4B, g[blue4]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED5R, g[red5]);
    SetPwm(PORT_LED5G, g[green5]);
    SetPwm(PORT_LED5B, g[blue5]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED6R, g[red6]);
    SetPwm(PORT_LED6G, g[green6]);
    SetPwm(PORT_LED6B, g[blue6]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED7R, g[red7]);
    SetPwm(PORT_LED7G, g[green7]);
    SetPwm(PORT_LED7B, g[blue7]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED8R, g[red8]);
    SetPwm(PORT_LED8G, g[green8]);
    SetPwm(PORT_LED8B, g[blue8]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
  }
  

  delay_ms(3000);
  if (PUSHBUTTON == 0) return 0;

  if (randomLed == 1) { red = red1; green = green1; blue = blue1; }
  if (randomLed == 2) { red = red2; green = green2; blue = blue2; }
  if (randomLed == 3) { red = red3; green = green3; blue = blue3; }
  if (randomLed == 4) { red = red4; green = green4; blue = blue4; }
  if (randomLed == 5) { red = red5; green = green5; blue = blue5; }
  if (randomLed == 6) { red = red6; green = green6; blue = blue6; }
  if (randomLed == 7) { red = red7; green = green7; blue = blue7; }
  if (randomLed == 8) { red = red8; green = green8; blue = blue8; }
    
  redNew = rand() % BRIGHTNESS_7;
  greenNew = rand() % BRIGHTNESS_7;
  blueNew = rand() % BRIGHTNESS_7;

  FadeOver(randomLed *3, randomLed *3 +1, randomLed *3 +2,
           red, green, blue,
           redNew, greenNew, blueNew,
           1000);
  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;

  if (randomLed == 1) { red1 = redNew; green1 = greenNew; blue1 = blueNew; }
  if (randomLed == 2) { red2 = redNew; green2 = greenNew; blue2 = blueNew; }
  if (randomLed == 3) { red3 = redNew; green3 = greenNew; blue3 = blueNew; }
  if (randomLed == 4) { red4 = redNew; green4 = greenNew; blue4 = blueNew; }
  if (randomLed == 5) { red5 = redNew; green5 = greenNew; blue5 = blueNew; }
  if (randomLed == 6) { red6 = redNew; green6 = greenNew; blue6 = blueNew; }
  if (randomLed == 7) { red7 = redNew; green7 = greenNew; blue7 = blueNew; }
  if (randomLed == 8) { red8 = redNew; green8 = greenNew; blue8 = blueNew; }

  return 0;
}

/***********************************************************************
*                                                                      *
*      03  program8                    parent = 01  main               *
*                                                                      *
*      Random colors fast running light.                               *
*                                                                      *
***********************************************************************/
int program8()
{
// Max brighness for this program.
#define BRIGHTNESS_8 33

  int i;

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }

  for (i = 0; i < NBR_LEDS; i += 3)
  {
    SetPwm(i, g[rand() % BRIGHTNESS_8]);
    SetPwm(i +1, g[rand() % BRIGHTNESS_8]);
    SetPwm(i +2, g[rand() % BRIGHTNESS_8]);
    delay_ms(10);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(i, 0);
    SetPwm(i +1, 0);
    SetPwm(i +2, 0);
    delay_ms(10);
    if (PUSHBUTTON == 0) return 0;
  }

  return 0;

}



/***********************************************************************
*                                                                      *
*      03  program9                    parent = 01  main               *
*                                                                      *
*      Leds on 1 by 1 randomly, then off 1 by 1 randomly, fast.        *
*                                                                      *
***********************************************************************/
int program9()
{
// Max brighness for this program.
#define BRIGHTNESS_8 33

  int i, j, v1, v2, nr, temp;
  int shuffle[] = {0, 1, 2, 3, 4, 5, 6, 7};

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }

  for (i = 0; i < 100; i++)
  {
    v1 = rand() % 8;
    v2 = rand() % 8;
    temp = shuffle[v1];
    shuffle[v1] = shuffle[v2];
    shuffle[v2] = temp;
  }
  for (i = 0; i < 8; i++)
  {
    nr = shuffle[i];
    /*
    SetPwm(nr *3, rand() % BRIGHTNESS_8);
    SetPwm(nr *3 +1, rand() % BRIGHTNESS_8);
    SetPwm(nr *3 +2, rand() % BRIGHTNESS_8);
    */

    FadeOver(nr *3, nr *3 +1, nr *3 +2,
             0, 0, 0,
             rand() % BRIGHTNESS_8, rand() % BRIGHTNESS_8, rand() % BRIGHTNESS_8,
             20);
    if (PUSHBUTTON == 0) return 0;
    delay_ms(20);
    if (PUSHBUTTON == 0) return 0;
  }

  delay_ms(200);
  if (PUSHBUTTON == 0) return 0;

  for (i = 0; i < 100; i++)
  {
    v1 = rand() % 8;
    v2 = rand() % 8;
    temp = shuffle[v1];
    shuffle[v1] = shuffle[v2];
    shuffle[v2] = temp;
  }
  for (i = 0; i < 8; i++)
  {
    nr = shuffle[i];
    SetPwm(nr *3, 0);
    SetPwm(nr *3 +1, 0);
    SetPwm(nr *3 +2, 0);
    delay_ms(20);
    if (PUSHBUTTON == 0) return 0;
  }


  return 0;

}



/***********************************************************************
*                                                                      *
*      03  program10                   parent = 01  main               *
*                                                                      *
*      Exploding effect. One LED on, another LED on until all LEDs     *
*      are on (increasing speed). Then wait a while and put all LEDs   *
*      off.                                                            *
*                                                                      *
***********************************************************************/
int program10()
{
// Max brighness for this program.
#define BRIGHTNESS_10 34

  int i, t, v1, v2, nr, temp;
  int shuffle[] = {0, 1, 2, 3, 4, 5, 6, 7};

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }

  for (i = 0; i < 100; i++)
  {
    v1 = rand() % 8;
    v2 = rand() % 8;
    temp = shuffle[v1];
    shuffle[v1] = shuffle[v2];
    shuffle[v2] = temp;
  }
  for (i = 0, t = 50; i < 8; i++, t -= 6)
  {
    nr = shuffle[i];
    SetPwm(nr *3, rand() % BRIGHTNESS_10);
    SetPwm(nr *3 +1, rand() % BRIGHTNESS_10);
    SetPwm(nr *3 +2, rand() % BRIGHTNESS_10);

    delay_ms(t);
    if (PUSHBUTTON == 0) return 0;
  }

  delay_ms(1000);
  if (PUSHBUTTON == 0) return 0;
  AllLedsOff();
  delay_ms(600);


  return 0;

}


/***********************************************************************
*                                                                      *
*      03  program11                   parent = 01  main               *
*                                                                      *
*      Mostly white. One by one (randomly) fading over to red, green,  *
*      blue and back to white.                                         *
*      off.                                                            *
***********************************************************************/
int program11()
{
// Max brighness for this program.
#define BRIGHTNESS_11 33

  int i, t, v1, v2, nr, temp;
  int shuffle[] = {0, 1, 2, 3, 4, 5, 6, 7};

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
  }

  for (i = 0; i < NBR_LEDS; i++)
    SetPwm(i, BRIGHTNESS_11);

  for (i = 0; i < 100; i++)
  {
    v1 = rand() % 8;
    v2 = rand() % 8;
    temp = shuffle[v1];
    shuffle[v1] = shuffle[v2];
    shuffle[v2] = temp;
  }
  for (i = 0; i < 8; i++)
  {
    nr = shuffle[i];

    FadeOver(nr *3, nr *3 +1, nr *3 +2,
             BRIGHTNESS_11, BRIGHTNESS_11, BRIGHTNESS_11,
             BRIGHTNESS_11, 0, 0,
             1000);
    if (PUSHBUTTON == 0) return 0;
    delay_ms(20);
    if (PUSHBUTTON == 0) return 0;
    FadeOver(nr *3, nr *3 +1, nr *3 +2,
             BRIGHTNESS_11, 0, 0,
             0, BRIGHTNESS_11, 0,
             1000);
    if (PUSHBUTTON == 0) return 0;
    delay_ms(20);
    if (PUSHBUTTON == 0) return 0;
    FadeOver(nr *3, nr *3 +1, nr *3 +2,
             0, BRIGHTNESS_11, 0,
             0, 0, BRIGHTNESS_11,
             1000);
    if (PUSHBUTTON == 0) return 0;
    delay_ms(20);
    if (PUSHBUTTON == 0) return 0;
    FadeOver(nr *3, nr *3 +1, nr *3 +2,
             0, 0, BRIGHTNESS_11,
             BRIGHTNESS_11, BRIGHTNESS_11, BRIGHTNESS_11,
             1000);
    if (PUSHBUTTON == 0) return 0;
    delay_ms(20);
    if (PUSHBUTTON == 0) return 0;
  }

  delay_ms(600);
  if (PUSHBUTTON == 0) return 0;


  return 0;

}


/***********************************************************************
*                                                                      *
*      03  program12                   parent = 01  main               *
*                                                                      *
*      Random colors and random intensity for each LED, 1 by 1,        *
*      randomly choosen and with fade-over effect, exactly like        *
*      program7 but very dim.                                          *
*                                                                      *
***********************************************************************/
int program12()
{
// Max brighness for this program.
#define BRIGHTNESS_12 9

  int randomLed;

  randomLed = rand() % 8;

  // Do first-time initialisations only the first time we are called.
  if (firstTime)
  {
    firstTime = 0;
    AllLedsOff();
    red1 = rand() % BRIGHTNESS_12;
    green1 = rand() % BRIGHTNESS_12;
    blue1 = rand() % BRIGHTNESS_12;
    red2 = rand() % BRIGHTNESS_12;
    green2 = rand() % BRIGHTNESS_12;
    blue2 = rand() % BRIGHTNESS_12;
    red3 = rand() % BRIGHTNESS_12;
    green3 = rand() % BRIGHTNESS_12;
    blue3 = rand() % BRIGHTNESS_12;
    red4 = rand() % BRIGHTNESS_12;
    green4 = rand() % BRIGHTNESS_12;
    blue4 = rand() % BRIGHTNESS_12;
    red5 = rand() % BRIGHTNESS_12;
    green5 = rand() % BRIGHTNESS_12;
    blue5 = rand() % BRIGHTNESS_12;
    red6 = rand() % BRIGHTNESS_12;
    green6 = rand() % BRIGHTNESS_12;
    blue6 = rand() % BRIGHTNESS_12;
    red7 = rand() % BRIGHTNESS_12;
    green7 = rand() % BRIGHTNESS_12;
    blue7 = rand() % BRIGHTNESS_12;
    red8 = rand() % BRIGHTNESS_12;
    green8 = rand() % BRIGHTNESS_12;
    blue8 = rand() % BRIGHTNESS_12;

    SetPwm(PORT_LED1R, g[red1]);
    SetPwm(PORT_LED1G, g[green1]);
    SetPwm(PORT_LED1B, g[blue1]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED2R, g[red2]);
    SetPwm(PORT_LED2G, g[green2]);
    SetPwm(PORT_LED2B, g[blue2]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED3R, g[red3]);
    SetPwm(PORT_LED3G, g[green3]);
    SetPwm(PORT_LED3B, g[blue3]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED4R, g[red4]);
    SetPwm(PORT_LED4G, g[green4]);
    SetPwm(PORT_LED4B, g[blue4]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED5R, g[red5]);
    SetPwm(PORT_LED5G, g[green5]);
    SetPwm(PORT_LED5B, g[blue5]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED6R, g[red6]);
    SetPwm(PORT_LED6G, g[green6]);
    SetPwm(PORT_LED6B, g[blue6]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED7R, g[red7]);
    SetPwm(PORT_LED7G, g[green7]);
    SetPwm(PORT_LED7B, g[blue7]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
    SetPwm(PORT_LED8R, g[red8]);
    SetPwm(PORT_LED8G, g[green8]);
    SetPwm(PORT_LED8B, g[blue8]);
    delay_ms(500);
    if (PUSHBUTTON == 0) return 0;
  }


  delay_ms(3000);
  if (PUSHBUTTON == 0) return 0;

  if (randomLed == 1) { red = red1; green = green1; blue = blue1; }
  if (randomLed == 2) { red = red2; green = green2; blue = blue2; }
  if (randomLed == 3) { red = red3; green = green3; blue = blue3; }
  if (randomLed == 4) { red = red4; green = green4; blue = blue4; }
  if (randomLed == 5) { red = red5; green = green5; blue = blue5; }
  if (randomLed == 6) { red = red6; green = green6; blue = blue6; }
  if (randomLed == 7) { red = red7; green = green7; blue = blue7; }
  if (randomLed == 8) { red = red8; green = green8; blue = blue8; }

  redNew = rand() % BRIGHTNESS_12;
  greenNew = rand() % BRIGHTNESS_12;
  blueNew = rand() % BRIGHTNESS_12;

  FadeOver(randomLed *3, randomLed *3 +1, randomLed *3 +2,
           red, green, blue,
           redNew, greenNew, blueNew,
           1000);
  delay_ms(500);
  if (PUSHBUTTON == 0) return 0;

  if (randomLed == 1) { red1 = redNew; green1 = greenNew; blue1 = blueNew; }
  if (randomLed == 2) { red2 = redNew; green2 = greenNew; blue2 = blueNew; }
  if (randomLed == 3) { red3 = redNew; green3 = greenNew; blue3 = blueNew; }
  if (randomLed == 4) { red4 = redNew; green4 = greenNew; blue4 = blueNew; }
  if (randomLed == 5) { red5 = redNew; green5 = greenNew; blue5 = blueNew; }
  if (randomLed == 6) { red6 = redNew; green6 = greenNew; blue6 = blueNew; }
  if (randomLed == 7) { red7 = redNew; green7 = greenNew; blue7 = blueNew; }
  if (randomLed == 8) { red8 = redNew; green8 = greenNew; blue8 = blueNew; }

  return 0;
}



/***********************************************************************
*                                                                      *
*      03  program31                   parent = 01  main               *
*                                                                      *
*      Combination of all other programs, random selection of program  *
*      for a random time.                                              *
*                                                                      *
***********************************************************************/
int program31()
{
  int i, t;
  int p;

  AllLedsOff();
  firstTime = 1;       // Make sure the new program will initialize.

  p = rand() % 31;     // Choose random program 0..30 (31 forbidden
                       // because this is ourselves...

  t = rand() % 25 + 25;    // Choose random time between 25 and 50 times.
  for (i = 0; i < t; i++)  // Execute random program for random time.
  {
    if (p == 0) program0();
    if (p == 1) program1();
    if (p == 2) program2();
    if (p == 3) program3();
    if (p == 4) program4();
    if (p == 5) program5();
    if (p == 6) program6();
    if (p == 7) program7();
    if (p == 8) program8();
    if (p == 9) program9();
    if (p == 10) program10();
    if (p == 11) program11();
    if (p == 12) program12();
    if (p == 13) program0();
    if (p == 14) program1();
    if (p == 15) program2();
    if (p == 16) program3();
    if (p == 17) program4();
    if (p == 18) program5();
    if (p == 19) program6();
    if (p == 20) program7();
    if (p == 21) program8();
    if (p == 22) program9();
    if (p == 23) program10();
    if (p == 24) program11();
    if (p == 25) program12();
    if (p == 26) program0();
    if (p == 27) program1();
    if (p == 28) program2();
    if (p == 29) program3();
    if (p == 30) program4();
    if (PUSHBUTTON == 0) return 0;
  }

  return 0;
}



