/***********************************************************************
*                                                                      *
*      Include files.                                                  *
*                                                                      *
***********************************************************************/
/* Device header file */
#if defined(__PIC24E__)
  #include <p24Exxxx.h>
#elif defined (__PIC24F__)
  #include <p24Fxxxx.h>
#elif defined(__PIC24H__)
  #include <p24Hxxxx.h>
#endif

#include <stdint.h>        /* Includes uint16_t definition */
#include <stdbool.h>       /* Includes true/false definition */

/***********************************************************************
*                                                                      *
*      Trap function prototypes.                                       *
*                                                                      *
***********************************************************************/

// <Other function prototypes for debugging trap code may be inserted here>

// Use if INTCON2 ALTIVT=1
void __attribute__((interrupt, no_auto_psv)) _OscillatorFail(void);
void __attribute__((interrupt, no_auto_psv)) _AddressError(void);
void __attribute__((interrupt, no_auto_psv)) _StackError(void);
void __attribute__((interrupt, no_auto_psv)) _MathError(void);

#if defined(__PIC24F__) || defined(__PIC24H__)

// Use if INTCON2 ALTIVT=0
void __attribute__((interrupt, no_auto_psv)) _AltOscillatorFail(void);
void __attribute__((interrupt, no_auto_psv)) _AltAddressError(void);
void __attribute__((interrupt, no_auto_psv)) _AltStackError(void);
void __attribute__((interrupt, no_auto_psv)) _AltMathError(void);

#endif

// Default interrupt handler
void __attribute__((interrupt, no_auto_psv)) _DefaultInterrupt(void);

#if defined(__PIC24E__)

/* These are additional traps in the 24E family.  Refer to the PIC24E
migration guide.  There are no Alternate Vectors in the 24E family. */
void __attribute__((interrupt, no_auto_psv)) _HardTrapError(void);
void __attribute__((interrupt, no_auto_psv)) _DMACError(void);
void __attribute__((interrupt, no_auto_psv)) _SoftTrapError(void);

#endif

/***********************************************************************
*                                                                      *
* Trap Handling                                                        *
*                                                                      *
* These trap routines simply ensure that the device continuously loops *
* within each routine.  Users who actually experience one of these     *
* traps can add code to handle the error.  Some basic examples for     *
* trap code, including assembly routines that process trap sources,    *
* are available at www.microchip.com/codeexamples                      *
*                                                                      *
***********************************************************************/

// Primary (non-alternate) address error trap function declarations.
void __attribute__((interrupt, no_auto_psv)) _OscillatorFail(void)
{
  INTCON1bits.OSCFAIL = 0;        /* Clear the trap flag */
  while (1);
}

void __attribute__((interrupt, no_auto_psv)) _AddressError(void)
{
  INTCON1bits.ADDRERR = 0;        /* Clear the trap flag */
  while (1);
}
void __attribute__((interrupt, no_auto_psv)) _StackError(void)
{
  INTCON1bits.STKERR = 0;         /* Clear the trap flag */
  while (1);
}

void __attribute__((interrupt, no_auto_psv)) _MathError(void)
{
  INTCON1bits.MATHERR = 0;        /* Clear the trap flag */
  while (1);
}

#if defined(__PIC24F__) || defined(__PIC24H__)

// Alternate address error trap function declarations.
void __attribute__((interrupt, no_auto_psv)) _AltOscillatorFail(void)
{
  INTCON1bits.OSCFAIL = 0;        // Clear the trap flag.
  while (1);
}

void __attribute__((interrupt, no_auto_psv)) _AltAddressError(void)
{
  INTCON1bits.ADDRERR = 0;        // Clear the trap flag
  while (1);
}

void __attribute__((interrupt, no_auto_psv)) _AltStackError(void)
{
  INTCON1bits.STKERR = 0;         // Clear the trap flag
  while (1);
}

void __attribute__((interrupt, no_auto_psv)) _AltMathError(void)
{
  INTCON1bits.MATHERR = 0;        /* Clear the trap flag */
  while (1);
}

#endif

/***********************************************************************
*                                                                      *
* Default Interrupt Handler                                            *
*                                                                      *
* This executes when an interrupt occurs for an interrupt source with  *
* an improperly defined or undefined interrupt handling routine.       *
*                                                                      *
***********************************************************************/
void __attribute__((interrupt, no_auto_psv)) _DefaultInterrupt(void)
{
  while (1);
}

#if defined(__PIC24E__)

/* These traps are new to the PIC24E family.  Refer to the device Interrupt
chapter of the FRM to understand trap priority. */
void __attribute__((interrupt, no_auto_psv)) _HardTrapError(void)
{
  while (1);
}
void __attribute__((interrupt, no_auto_psv)) _DMACError(void)
{
  while (1);
}
void __attribute__((interrupt, no_auto_psv)) _SoftTrapError(void)
{
  while (1);
}

#endif
