/***********************************************************************
*                                                                      *
*      Include files.                                                  *
*                                                                      *
***********************************************************************/
/* Device header file */
#if defined(__PIC24E__)
  #include <p24Exxxx.h>
#elif defined (__PIC24F__)
  #include <p24Fxxxx.h>
#elif defined(__PIC24H__)
  #include <p24Hxxxx.h>
#endif

#include <stdint.h>          /* For uint32_t definition */
#include <stdbool.h>         /* For true/false definition */

#include "user.h"            /* variables/params used by user.c */
#include "Utility.h"

/***********************************************************************
*                                                                      *
*      User functions.                                                 *
*                                                                      *
***********************************************************************/

// <Initialize variables in user.h and insert code for user algorithms.>





/***********************************************************************
*                                                                      *
*      03  InitApp                     parent = 01  main               *
*                                                                      *
***********************************************************************/
// Initialize User Ports/Peripherals/Project here.
void InitApp(void)
{
  int i = 0;

  // Setup analog functionality and port direction.
  TRISBbits.TRISB10 = 0;
  TRISCbits.TRISC9 = 0;
  TRISCbits.TRISC8 = 0;
  TRISBbits.TRISB9 = 0;
  TRISCbits.TRISC6 = 0;
  TRISCbits.TRISC7 = 0;
  TRISCbits.TRISC5 = 0;
  TRISBbits.TRISB7 = 0;
  TRISBbits.TRISB8 = 0;
  TRISAbits.TRISA4 = 0;
  TRISCbits.TRISC3 = 0;
  TRISCbits.TRISC4 = 0;
  TRISCbits.TRISC1 = 0;
  TRISCbits.TRISC2 = 0;
  TRISAbits.TRISA2 = 0;
  TRISBbits.TRISB2 = 0;
  TRISBbits.TRISB3 = 0;
  TRISCbits.TRISC0 = 0;
  TRISAbits.TRISA1 = 0;
  TRISBbits.TRISB0 = 0;
  TRISBbits.TRISB1 = 0;
  TRISBbits.TRISB14 = 0;
  TRISBbits.TRISB15 = 0;
  TRISAbits.TRISA0 = 0;

  TRISAbits.TRISA3 = 0;
  TRISAbits.TRISA7 = 0;
  TRISAbits.TRISA10 = 0;
  TRISBbits.TRISB12 = 0;
  TRISBbits.TRISB11 = 0;

  // Initialize peripherals.

  PUSHBUTTON_CN = 1;   // Enable pull-up for the pushbutton input.

  // Self test.
  IND1 = 1;
  delay_ms(100);
  IND1 = 0;
  i = 0;
  IND2 = 1;
  delay_ms(100);
  IND2 = 0;
  i = 0;
  IND3 = 1;
  delay_ms(100);
  IND3 = 0;
  i = 0;
  IND4 = 1;
  delay_ms(100);
  IND4 = 0;
  i = 0;
  IND5 = 1;
  delay_ms(100);
  IND5 = 0;

#define H 100
  LED1R = 1;
  delay_ms(H);
  LED1R = 0;
  delay_ms(H);
  LED1G = 1;
  delay_ms(H);
  LED1G = 0;
  delay_ms(H);
  LED1B = 1;
  delay_ms(H);
  LED1B = 0;
  delay_ms(H);
  LED2R = 1;
  delay_ms(H);
  LED2R = 0;
  delay_ms(H);
  LED2G = 1;
  delay_ms(H);
  LED2G = 0;
  delay_ms(H);
  LED2B = 1;
  delay_ms(H);
  LED2B = 0;
  delay_ms(H);
  LED3R = 1;
  delay_ms(H);
  LED3R = 0;
  delay_ms(H);
  LED3G = 1;
  delay_ms(H);
  LED3G = 0;
  delay_ms(H);
  LED3B = 1;
  delay_ms(H);
  LED3B = 0;
  delay_ms(H);
  LED7R = 1;
  delay_ms(H);
  LED7R = 0;
  delay_ms(H);
  LED7G = 1;
  delay_ms(H);
  LED7G = 0;
  delay_ms(H);
  LED7B = 1;
  delay_ms(H);
  LED7B = 0;
  delay_ms(H);
  LED8R = 1;
  delay_ms(H);
  LED8R = 0;
  delay_ms(H);
  LED8G = 1;
  delay_ms(H);
  LED8G = 0;
  delay_ms(H);
  LED8B = 1;
  delay_ms(H);
  LED8B = 0;
  delay_ms(H);

  for (i = 0; i < 10; i++)
  {
    LED1R = 1;
    delay_ms(1);
    LED1G = 1;
    delay_ms(1);
    LED1B = 1;
    delay_ms(100);
    LED1R = 0;
    delay_ms(1);
    LED1G = 0;
    delay_ms(1);
    LED1B = 0;
    delay_ms(100);
  }

  // Setup timer 1.
  _T1IP = 4;   // Timer1 interrupt priority set to 4.

  // Set TIMER1 to about 1 ms.
  TMR1 = TIM;
  T1CON = 0x8000;      // Timer1 is enabled, prescaler 1:1, internal clock.

  // Allow TIMER1 interrupts.
  _T1IF = 0;
  _T1IE = 1;

}


/***********************************************************************
*                                                                      *
*      09  SetPwm                      parents are many                *
*                                                                      *
*      Sets the pwm value for a particular pwm channel.                *
*                                                                      *
***********************************************************************/
int SetPwm(int port, int value)
{
  int p, v;

  p = port % NBR_LEDS;
  v = value % 1024;

  if (p < 0) p = 0;
  if (v < 0) v = 0;
  
  pwm[p] = v;

  return 0;
}

/***********************************************************************
*                                                                      *
*      09  SetColor                    parents are many                *
*                                                                      *
*      Sets a specific LED to a specific color.                        *
*                                                                      *
*      led: 1..8 the led for which color must be set                   *
*      color: 0x000000..0xffffff the desired rgb color                 *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/
int SetColor(int led, long color)
{
  int portR, portG, portB;
  int red, green, blue;
  
  portR = (led -1) * 3;
  portG = portR +1;
  portB = portG +1;

  red = (color / 0x10000) / 4;             // red is now 0..63
  green = ((color / 0x100) % 0x100) / 4;   // green is now 0..63
  blue = (color % 0x100) / 4;              // blue is now 0..63
  
  SetPwm(portR, g[red]);
  SetPwm(portG, g[green]);
  SetPwm(portB, g[blue]);
  
  return 0;
}

/***********************************************************************
*                                                                      *
*      09  FadeOver                    parents are many                *
*                                                                      *
*      Fade over from 1 rgb value to another, on a specific pwm        *
*      channel. Desired time for the fade-over should be specified in  *
*      milliseconds.                                                   *
*                                                                      *
***********************************************************************/
int FadeOver(int portR, int portG, int portB,
             int startR, int startG, int startB,
             int endR, int endG, int endB,
             int ms)
{
#if 0
  int steps = 150;
  double stepMS = 1;
  double red, green, blue;
  double stepR, stepG, stepB;
  double diff;
  int v1, v2;

  if (ms < 500) steps = 10;
  if (ms < 1000 && ms >= 500) steps = 50;
  if (ms >= 1000) steps = 150;

  stepMS = ms / steps;

  red = g[startR];
  green = g[startG];
  blue = g[startB];

  v1 = g[endR];
  v2 = g[startR];
  diff = v1 - v2;
  stepR = diff / (double)steps;

  v1 = g[endG];
  v2 = g[startG];
  diff = v1 - v2;
  stepG = diff / (double)steps;

  v1 = g[endB];
  v2 = g[startB];
  diff = v1 - v2;
  stepB = diff / (double)steps;


  int i;
  for (i = 0; i < steps; i++)
  {
    SetPwm(portR, ((int)red)) % 1024;
    SetPwm(portR, ((int)green)) % 1024;
    SetPwm(portR, ((int)blue)) % 1024;

    red += stepR;
    green += stepG;
    blue += stepB;

    delay_ms((int)stepMS);
    ClrWdt();
    if (PUSHBUTTON == 0) return 1;
  }

  return 0;
#endif
  int steps = 150;
  double stepMS = 1;
  double red, green, blue;
  double stepR, stepG, stepB;
  double diff;
  int val;

  if (ms < 500) steps = 10;
  if (ms < 1000 && ms >= 500) steps = 50;
  if (ms >= 1000) steps = 150;

  stepMS = ms / steps;

  red = startR;
  green = startG;
  blue = startB;

  diff = endR - startR;
  stepR = diff / ((double)steps);
  diff = endG - startG;
  stepG = diff / ((double)steps);
  diff = endB - startB;
  stepB = diff / ((double)steps);

  int i;
  for (i = 0; i < steps; i++)
  {
    val = (int)red;
    if (val < 0) val = 0;
    SetPwm(portR, g[val]);
    val = (int)green;
    if (val < 0) val = 0;
    SetPwm(portG, g[val]);
    val = (int)blue;
    if (val < 0) val = 0;
    SetPwm(portB, g[val]);

    red += stepR;
    green += stepG;
    blue += stepB;

    delay_ms((int)stepMS);
    ClrWdt();
    if (PUSHBUTTON == 0) return 1;
  }

  return 0;

}

/***********************************************************************
*                                                                      *
*      09  AllLedsOff                  parents are many                *
*                                                                      *
*      Turns all pwm channels to 0.                                    *
*                                                                      *
***********************************************************************/
int AllLedsOff()
{
  SetPwm(PORT_LED1R, 0);
  SetPwm(PORT_LED1G, 0);
  SetPwm(PORT_LED1B, 0);
  SetPwm(PORT_LED2R, 0);
  SetPwm(PORT_LED2G, 0);
  SetPwm(PORT_LED2B, 0);
  SetPwm(PORT_LED3R, 0);
  SetPwm(PORT_LED3G, 0);
  SetPwm(PORT_LED3B, 0);
  SetPwm(PORT_LED4R, 0);
  SetPwm(PORT_LED4G, 0);
  SetPwm(PORT_LED4B, 0);
  SetPwm(PORT_LED5R, 0);
  SetPwm(PORT_LED5G, 0);
  SetPwm(PORT_LED5B, 0);
  SetPwm(PORT_LED6R, 0);
  SetPwm(PORT_LED6G, 0);
  SetPwm(PORT_LED6B, 0);
  SetPwm(PORT_LED7R, 0);
  SetPwm(PORT_LED7G, 0);
  SetPwm(PORT_LED7B, 0);
  SetPwm(PORT_LED8R, 0);
  SetPwm(PORT_LED8G, 0);
  SetPwm(PORT_LED8B, 0);

  ClrWdt();

  return 0;
}

/***********************************************************************
*                                                                      *
*      09  delay_ms                    parents are many                *
*                                                                      *
*      Delays the requested number of milliseconds but will            *
*      return immediately if the button is pressed.                    *
*                                                                      *
***********************************************************************/
void delay_ms(int d)
{
  int i, j;

  //if (!DEBUG) Delay(d);
  ClrWdt();

  for (i = 0; i < d && !DEBUG; i++)
    for (j = 0; j < 3600; j++)
    {
      ClrWdt();
      if (PUSHBUTTON == 0) return;
    }
}


